/******************************************************************************
    CONTROLE REMOTO DE 8 CANAIS POR TONS DTMF

    C/ microcontrolador PIC16F676

    Receptor para sinais DTMF com 8 canais acionados conforme a tecla DTMF.

    Direto, sem senhas - apertando uma tecla acionar a sada correspondente.

    Cada canal pode ser ajustado para modo pulso ou reteno individualmente
    

    Autor: Cludio Lrios

    Fontes de pesquisas:
    Baseado nos seguintes artigos publicados em Websites originalmente escritos
    em linguagem assembly  e sendo adaptado para linguagem C:

    Adaptaes de Tabelas e algoritimos Goertzel: Radu Constantinescu.
    Microchip Note Applications AN257 'DTMF Detection Using PIC18
    Microcontrollers' by Gaurang Kavaiya.
    DTMF Decoding Software with a PIC - by Wim de Vree PE1GRL - pe1grl@amsat.org
    Introduction: DTMF Touch Tone Decoder Using Microchip PIC Microprocessor by
    LargeMouthBass - https://www.instructables.com/member/LargeMouthBass/
    Black DTMF Decoding Algorithm - www.RomanBlack.com
    Agradecimentos a todos os autores destes artigos por compartilharem seus
    conhecimentos a todos que tem o desejo sincero de aprender.

    Compilador sugerido: CCS C V3.200


    PINAGEM DO PIC 16F676:

    PINO 12  =   (entrada) sinal DTMF
    PINO 10  =   rele tecla 1 (sada)
    PINO  9  =   rele tecla 2 (sada)
    PINO  8  =   rele tecla 3 (sada)
    PINO  7  =   rele tecla 4 (sada)
    PINO  6  =   rele tecla 5 (sada)
    PINO  5  =   rele tecla 6 (sada)
    PINO  13 =   rele tecla 7 (sada)
    PINO  11 =   rele tecla 8 (sada)
    PINO  2  =   xtal 20MHZ
    PINO  3  =   xtal 20MHZ
    PINO  1  =   VCC
    PINO 14  =   GND

    Uso didtico

    Blog Picsource
******************************************************************************/
//Escolha do usurio - Descomente ou comente conforme o tipo de sida desejada
//forma individualizada por canais.
// descomente para modo reteno ou deixe comentado para modo pulso

 //  #define tecla1_retencao
//  #define tecla2_retencao
//   #define tecla3_retencao
//   #define tecla4_retencao
    #define tecla5_retencao
    #define tecla6_retencao
   #define tecla7_retencao
    #define tecla8_retencao

//Aps alterar, este arquivo deve ser recompilado para obter o novo arquivo .hex
//que dever ser carregado no pic.
//==============================================================================
#include <16F676.h>
#use delay(clock=20000000)//frequencia do xtal
#fuses NOWDT,HS, NOCPD, NOPROTECT, NOMCLR, NOPUT, NOBROWNOUT

//bytes

#byte   FSR         = 0x04
#byte   PCLATH      = 0x0A
#byte   STATUS      = 0X03
#byte   TRISA       = 0x85
#byte   PORTA       = 0x05
#byte   TRISC       = 0x87
#byte   PORTC       = 0x07
#byte   TMR1H       = 0x0F
#byte   TMR1L       = 0x0E
#byte   TMR0        = 0X01
#byte   OPTION_REG  = 0x81
#byte   WPU         = 0x95
#byte   CMCON       = 0x19
#byte   ANSEL       = 0x91 //obs. diferente do pic12f675
#byte   ADRESH      = 0x1E
#byte   ADRESL      = 0x9E
#byte   T1CON       = 0X10
#byte   IOC         = 0x96
#byte   INTCON      = 0X0B
#byte   VRCON       = 0X99
#byte   ADCON0      = 0X1F
#byte   VRCON       = 0X99
#byte   FSR         = 0X04
#byte   INDF        = 0X00

//bit
#bit    GIE         = 0xb.7
#bit    PEIE        = 0xb.6
#bit    T0IE        = 0xb.5
#bit    TMR1IE      = 0x8C.0
#bit    TMR1IF      = 0xC.0
#bit    TMR1_ON     = 0x10.0
#bit    T0IF        = 0XB.2
#bit    TMR0_F0     = 0x01.0
#bit    C_OUT       = 0x19.6
#bit    GO_DONE     = 0X1F.1

//pinos de sada

#bit can1 = 0x7.0  //pino 10
#bit can2 = 0x7.1  //pino  9
#bit can3 = 0x7.2  //pino  8
#bit can4 = 0x7.3  //pino  7
#bit can5 = 0x7.4  //pino  6
#bit can6 = 0x7.5  //pino  5
#bit can7 = 0x5.0  //pino 13
#bit can8 = 0x5.2  //pino 11

#define N_AMOSTRAGENS 240
#define N_AMOSTRAGENS_X2 113 //1/2 amostragem - 7
#define LIMITE_MIN_X1L  60  //1/4 amostragem x1
#define LIMITE_MIN_X1H  48  //1/5 amostragem x1
#define LIMITE_MIN_X2L  37  //1/3 amostragem x2 - 3
#define LIMITE_MIN_X2H  37  //1/3 amostragem x2 - 3

#define TEMPO_DESL_PULSO 12//tempo mnimo para no ficar piscando a sada
#define TEMPO_DESL_RET 5//tempo mnimo para evitar alternancia c/ boto apertado

// variveis globais
char cnt_amostragem=0;//numero de amostragens
char mascara;//valor relativo a situao do pino 12 (sinal DTMF)
char tecla_ant=0;//guarda tecla anterior
char tecla;//recebe valor da tecla pressionada
char cnt1;//contadores de tempo auxiliares
char cnt2;
char cnt3;
char cnt4;
char cnt5;
char cnt6;
char cnt7;
char cnt8;

//flags
unsigned int1 fvez=0;// indica confirmao da tecla

#ifdef tecla1_retencao
unsigned int1 fc1=0;
#endif
#ifdef tecla2_retencao
unsigned int1 fc2=0;
#endif
#ifdef tecla3_retencao
unsigned int1 fc3=0;
#endif
#ifdef tecla4_retencao
unsigned int1 fc4=0;
#endif
#ifdef tecla5_retencao
unsigned int1 fc5=0;
#endif
#ifdef tecla6_retencao
unsigned int1 fc6=0;
#endif
#ifdef tecla7_retencao
unsigned int1 fc7=0;
#endif
#ifdef tecla8_retencao
unsigned int1 fc8=0;
#endif


/* Acumulador de 32 bytes para guardar valores dos senos e cossenos
das frequncia DTMF -> 697,770,852,941 (baixas) e 1209,1336,1447,1633 (altas)
obtidas de 2 tabelas na flash: tab_low_band e tab_high_band.
Os primeiros 16 bytes contero valores de senos e cossenos para freq x 1.
Os 16 bytes seguintes contero valores de senos e cossenos para freq x 2.
Acesso ao array por nmero 'par' obteremos os valores de senos e 'impares' os
valores de cossenos. Veja abaixo:

sin_cos[0]; -> seno 697 freq x1
sin_cos[1]; -> cosseno 697 freq x1
sin_cos[2]; -> seno 770 freq x1
sin_cos[3]; -> cosseno 770 freq x1
sin_cos[4]; -> seno 852 freq x1
sin_cos[5]; -> cosseno 852 freq x1
sin_cos[6]; -> seno 941 freq x1
sin_cos[7]; -> cosseno 941 freq x1

sin_cos[8]; -> seno 1209 freq x1
sin_cos[9]; -> cosseno 1209 freq x1
sin_cos[10]; -> seno 1336 freq x1
sin_cos[11]; -> cosseno 1336 freq x1
sin_cos[12]; -> seno 1447 freq x1
sin_cos[13]; -> cosseno 1447 freq x1
sin_cos[14]; -> seno 1633 freq x1
sin_cos[15]; -> cosseno 1633 freq x1

sin_cos[16]; -> seno 697 freq x2
sin_cos[17]; -> cosseno 697 freq x2
sin_cos[18]; -> seno 770 freq x2
sin_cos[19]; -> cosseno 770 freq x2
sin_cos[20]; -> seno 852 freq x2
sin_cos[21]; -> cosseno 852 freq x2
sin_cos[22]; -> seno 941 freq x2
sin_cos[23]; -> cosseno 941 freq x2

sin_cos[24]; -> seno 1209 freq x2
sin_cos[25]; -> cosseno 1209 freq x2
sin_cos[26]; -> seno 1336 freq x2
sin_cos[27]; -> cosseno 1336 freq x2
sin_cos[28]; -> seno 1447 freq x2
sin_cos[29]; -> cosseno 1447 freq x2
sin_cos[30]; -> seno 1633 freq x2
sin_cos[31]; -> cosseno 1633 freq x2
*/
unsigned int8 sin_cos[32];//array de senos e cossenos


/* tabela de mscaras para as amostragens 697,770,852,941 hz
   Obs. Cada bit tem informao para cada uma das 4 frequncias DTMF inferiores,
   sendo:
   b7= sin 697, b6=cos697, b5=sin770, b4=cos770,
   b3=sin852, b2=cos852, b1=sin941, b0=cos941 */
const char tab_low_band[]={
 0x55,0xff,0xff,0xfa,0xaa,0xaa,0xa0,0x80,0x01,0x05,
 0x15,0x57,0x5f,0x7e,0x7e,0xfa,0xe8,0xe8,0xa0,0xa1,
 0x81,0x87,0x07,0x1f,0x1e,0x5e,0x78,0x78,0x78,0xe1,
 0xe1,0xe1,0x87,0x87,0x86,0x9e,0x1e,0x1c,0x18,0x79,
 0x79,0x71,0x63,0xe7,0xe7,0xc6,0x8e,0x8c,0x9c,0x98,
 0x19,0x39,0x33,0x73,0x63,0x66,0x66,0xe6,0xcc,0xcc,
 0x8d,0x99,0x99,0x9b,0x33,0x32,0x36,0x66,0x64,0x6c,
 0x4c,0xcd,0xc9,0xdb,0x9b,0x93,0x92,0xb2,0x36,0x36,
 0x26,0x6d,0x6d,0x4d,0x4b,0xcb,0xd2,0xd2,0x92,0xb4,
 0xb4,0xb5,0x2d,0x2d,0x2f,0x6b,0x4a,0x4a,0x42,0xd0,
 0xd0,0xd4,0xb5,0xb5,0xbf,0xaf,0x2f,0x2a,0x0a,0x40,
 0x40,0x50,0x55,0xd5,0xd5,0xff,0xbf,0xbe,0xaa,0xaa,
 0x28,0x00,0x01,0x41,0x55,0x57,0x57,0xff,0xfe,0xfa,
 0xf8,0xa8,0xa0,0xa1,0x01,0x07,0x07,0x57,0x5e,0x5e,
 0x7e,0xf8,0xf8,0xe9,0xa1,0xa1,0x83,0x87,0x06,0x06,
 0x1e,0x5c,0x58,0x78,0x79,0xf1,0xe3,0xe3,0xa7,0x96,
 0x86,0x8c,0x1c,0x1c,0x19,0x79,0x79,0x73,0x73,0xe2,
 0xe6,0xe6,0x8c,0x8c,0x8d,0x99,0x19,0x1b,0x33,0x73,
 0x72,0x66,0x64,0xe4,0xcc,0xcd,0x8d,0x8b,0x9b,0x9b,
 0x12,0x32,0x34,0x74,0x64,0x6d,0x6d,0xcd,0xcb,0xcb,
 0x9a,0x92,0x92,0xb0,0x34,0x35,0x35,0x6d,0x6f,0x6f,
 0x4b,0xca,0xca,0xd0,0x90,0x94,0xb5,0xb5,0x3f,0x2f,
 0x2f,0x6a,0x4a,0x4a,0x40,0xc0,0xd1,0xd5,0x95,0xb7,
 0xbf,0xbe,0x2e,0x2a,0x28,0x40,0x40,0x41,0x55,0xd7,
 0xd7,0xff,0xbe,0xbe,0xb8,0xa8,0x28,0x21,0x01,0x41,
 0x47,0x57,0x56,0xde,0xfe
};

/* tabela de mscaras para as amostragens 1209,1336,1447,1633 hz
   Obs. Cada bit tem informao para cada uma das 4 frequncias DTMF superiores,
   sendo:
   b7= sin 1209, b6=cos1209, b5=sin1336, b4=cos1336,
   b3=sin1477, b2=cos1477, b1=sin1633, b0=cos1633 */
const int8 tab_high_band[]={
 0x55,0xff,0xea,0xa8,0x80,0x05,0x17,0x5f,0x7e,0xf8,
 0xe0,0x81,0x87,0x17,0x1e,0x78,0x78,0xe1,0xe3,0x87,
 0x8e,0x1c,0x38,0x79,0x63,0xe7,0xc6,0x8c,0x9c,0x19,
 0x33,0x73,0x66,0xc4,0xcc,0x99,0x9b,0x33,0x32,0x64,
 0x6c,0xcd,0xcb,0x93,0xb2,0x34,0x24,0x6d,0x4b,0xcb,
 0xd2,0x90,0xb4,0x2d,0x2f,0x4b,0x4a,0xd0,0xd4,0xb5,
 0xbf,0x2f,0x2a,0x40,0x50,0xd5,0xf7,0xbf,0xaa,0x28,
 0x00,0x41,0x57,0xdf,0xfe,0xa8,0xa8,0x01,0x07,0x57,
 0x5e,0xf8,0xf8,0xa1,0xa3,0x07,0x1e,0x5c,0x78,0xf9,
 0xe3,0xa7,0x86,0x0c,0x1c,0x59,0x73,0xe3,0xe6,0x84,
 0x8c,0x19,0x1b,0x73,0x72,0x64,0xcc,0xcd,0x9b,0x9b,
 0x32,0x34,0x64,0x6d,0xcf,0xcb,0x92,0xb0,0x34,0x25,
 0x6f,0x4b,0xca,0xd0,0x94,0xb5,0x3f,0x2f,0x4a,0x40,
 0xd0,0xd5,0xb7,0xbf,0x2a,0x28,0x40,0x41,0xd7,0xff,
 0xbe,0xa8,0x28,0x01,0x47,0x57,0xde,0xfc,0xb8,0xa1,
 0x03,0x06,0x56,0x5c,0xf9,0xf9,0xa3,0xa2,0x06,0x1c,
 0x5d,0x79,0xf3,0xe2,0xa6,0x84,0x0d,0x19,0x5b,0x72,
 0xe2,0xe4,0x8d,0x8d,0x1b,0x1a,0x72,0x74,0xe5,0xed,
 0x8f,0x9a,0x12,0x30,0x75,0x65,0xef,0xca,0x8a,0x90,
 0x11,0x35,0x6f,0x6e,0xca,0xc8,0x91,0x95,0x37,0x3e,
 0x6e,0x68,0xc1,0xd1,0x97,0xbe,0x3e,0x28,0x69,0x41,
 0x47,0xd6,0xfe,0xbc,0xa9,0x21,0x03,0x46,0x56,0xdc,
 0xf9,0xb9,0xa3,0x02,0x06,0x5c,0x5d,0xf9,0xfb,0xa2,
 0xa6,0x04,0x0d,0x5d,0x7b,0xf2,0xe2,0xa4,0x85,0x0d,
 0x1b,0x5a,0x72,0xf4,0xe5,0x8d,0x8f,0x1a,0x12,0x70,
 0x75,0xe5,0xef,0x8a,0x8a
};


/* valores de magnitudes referentes a cada par de frequncias de tons DTMF
   sendo :
   bit7=1633  bit6=1447 bit5=1336 bit4=1209
   bit3=941   bit2=852  bit1=770  bit0=697
   Obs: O bit setado indica presena da respectiva frequncia conforme acima */
const char tab_magnitudes[]={
 //0   1    2    3     4   5    6    7    8    9    *   #     A    B    C    D
  40, 17,  33,  65,   18,  34,  66, 20,   36,  68,  24, 72,  129, 130,  132, 136
};

/* Tabela p/ retorno do valor da tecla apertada (retorna conforme posies da
tabela de magnitudes */
const char tab_teclas[]={ //                       *      #
'0','1','2','3', '4', '5',  '6', '7', '8', '9', 0x2a, 0x23,'A','B', 'C', 'D'

};

/*==============================================================================
 ROTINA DE INTERRUPO DO TIMER 0 - (executada a cada 102,4 us)
 Obs. O modo de interrupo global no tem salvamento automtico de valores dos
 principais registradores e nem apaga flag de overflow do timer0. Porque usa-la?
 O motivo  que a rotina '#int_timer0' gasta muito tempo salvando registradores
 desnecessrios aos objetivos desta montagem.
 Para funcionar, a rotina de interrupo do timer0 no pode exceder 102,4 us.
 A rotina atual gasta 96,8 us.
 Note que as rotinas usam o FSR diretamente como ponteiro. Tambm foi priorizado
 o uso de rotinas curtas. Cada rotina foi reduzida ao mximo para no exceder
 o tempo mximo de execuo.
 Se fosse usado '#int_timer0' e o modo padro de rotinas C, este tempo passaria
 tranquilamente dos 160 us.
 Cuidado especial deve ser dado a este fato caso venha a migrar para outros
 tipos de microcontroladores.
 Tambm tem que ser usado a frequncia de 20mhz (frequncia mximo deste PIC).
==============================================================================*/
#INT_GLOBAL
void isr_timer0(){//interrupo do timer 0 (conta tempo para amostragens)
//variveis locais
char cnt_amostragem2x;
char modelo_freq;//guarda modelo da amostragem para uma dada frequencia
char STATUS_TEMP,PCLATH_SAVE,FSR_SAVE,W_TEMP;

   #ASM
   movwf	 W_TEMP
	swapf	 STATUS,W
	clrf	 STATUS		//ENSURE BANK0 SAVE
	movwf	 STATUS_TEMP
	movf	 PCLATH,W	  //SAVE PCLATH
	movwf  PCLATH_SAVE
	movf	 FSR, W
	movwf	 FSR_SAVE
   #ENDASM

  /*testa bit de sada do comparador analgico que contm a informao do sinal
  DTMF (na entrada do pino 6).
  */
  mascara=0;//inicia zerado
  if(C_OUT)mascara--;//Se sada do comparador analgico  '1' ento mascara=0xff
  cnt_amostragem2x=cnt_amostragem<<1;//obtem posio para tabela x2

  /* Obtem valores diferentes de senos e cossenos das frequncias de tom para cada
  amostragem efetuada atravs de 2 tabelas na flash. Estes valores so mascarados
  conforme a situao da entrada de sinal DTMF (pino 12, via comparador analgico).
  */


  FSR= sin_cos; //aponta ponteiro para inicio do acumulador seno/cosseno 32
  modelo_freq=tab_low_band[cnt_amostragem] ^ mascara;//obtem valores das tabela low x1
  do{
    if(modelo_freq & 0x80)INDF--;//bit =1 ento decrementa acumulador apontado
    if(!(modelo_freq & 0x80))INDF++;//bit 0 ento incrementa acumulador apontado
    modelo_freq= modelo_freq <<1; //desloca para esquerda para mudar o bit
    FSR++;//muda o acumulador
    }while(FSR<sin_cos+8);//percorre 8 bytes

  modelo_freq=tab_high_band[cnt_amostragem] ^ mascara;//obtem valores das tabela high x1
  do{
    if(modelo_freq & 0x80)INDF--; //se 1 decrementa
    if(!(modelo_freq & 0x80))INDF++;
    modelo_freq= modelo_freq <<1; //desloca para esquerda
    FSR++;
  }while(FSR<sin_cos+16);//percorre 8 bytes


  if(cnt_amostragem<N_AMOSTRAGENS_X2){//limita quantidade de amostragens em frequncias 2x
     modelo_freq=tab_low_band[cnt_amostragem2x] ^ mascara;//obtem valores das tabela low x2
     do{
      if(modelo_freq & 0x80)INDF--; //se 1 decrementa
      if(!(modelo_freq & 0x80))INDF++; //se 0 incrementa
      modelo_freq= modelo_freq <<1; //desloca para esquerda
      FSR++;
     }while(FSR<sin_cos+24);//percorre 8 bytes
     modelo_freq=tab_high_band[cnt_amostragem2x] ^ mascara;//obtem valores das tabela high x2
     do{
      if(modelo_freq & 0x80)INDF--; //se 1 decrementa
      if(!(modelo_freq & 0x80))INDF++; //se 0 incrementa
      modelo_freq= modelo_freq <<1; //desloca para esquerda
      FSR++;
     }while(FSR<sin_cos+32);//percorre 8 bytes
   }


  if(++cnt_amostragem>=N_AMOSTRAGENS){//incrementa e se chegou no valor mximo
    T0IE=0;//Desliga interrupo do timer 0 e sossega at a rotina principal agir
  }

  T0IF=0;//reseta flag de overflow do timer0

  #asm //restabelece principais registradores com valores anteriores
  	 movf	   FSR_SAVE,W
	 movwf	FSR
	 movf	   PCLATH_SAVE,W
	 movwf	PCLATH		// RESTORE PCLATH
	 swapf	STATUS_TEMP,W
	 movwf	STATUS
	 swapf	W_TEMP,F
	 swapf	W_TEMP,W
  #endasm
}//fim da rotina de interrupo do timer 0


/*------------------------------------------------------------------------------
     ROTINA PARA ZERAR ARRAY COM VALORES DE SENOS E COSSENOS
-------------------------------------------------------------------------------*/
void zera_array(){
  for(FSR=sin_cos;FSR<=sin_cos+32;FSR++){
     INDF=0; //apaga array seno-cosseno
  }
}
/*------------------------------------------------------------------------------
     ROTINA PARA INICIAR AMOSTRAGENS
-------------------------------------------------------------------------------*/
void dispara_amostragens(){
  zera_array();//zera array de senos e cossenos
  TMR0=0;//zera timer 0
  cnt_amostragem=0;//reinicia contador de amostragem
  INTCON= 0B10100000; //liga interrupo do timer 0  e inicia nova amostragem
}


/*------------------------------------------------------------------------------
   ROTINA PARA OBTER OS TONS DTMF (SE PRESENTES NA ENTRADA DE SINAL - PINO 12)
-------------------------------------------------------------------------------*/

char decodifica_dtmf(){
char a,b,c,d;
char res_magx1,res_magx2;

   res_magx1=0;//registrador p/ resultados das magnitudes das 8 frequencias x 1
   res_magx2=0;//registrador p/ resultados das magnitudes das 8 frequencias x 2

   for(b=0;b<32;b++){

     d= sin_cos[b];
     if(d & 0x80)d = 0 - d; //obtem o valor absoluto do seno
     b++;
     c= sin_cos[b];
     if(c & 0x80)c = 0 - c; //obtem o valor absoluto do cosseno
     c= c + d;// soma seno e cosseno absolutos

     /*=========================================================================
     Rotina abaixo compara as magnitudes obtidas nos acumuladores com os
      limites mnimos e passando os valores aprovados p/ os bytes res_magx1 e
      res_magx2
    ========================================================================= */
     if(b<8){//quando b= 1,3,5,7  (low freq x1)
       res_magx1=res_magx1>>1;
       if(c>=LIMITE_MIN_X1L)res_magx1=res_magx1 | 0x80;//se c>limite
     }
     else if(b<16){//quando b= 9,11,13,15  (high freq x1)
       res_magx1=res_magx1>>1;
       if(c>=LIMITE_MIN_X1H)res_magx1=res_magx1 | 0x80;//se c>limite
     }

     else if(b<24){//quando b= 17,19,21,23 (low freq x2)
       res_magx2=res_magx2>>1;
       if(c<LIMITE_MIN_X2L)res_magx2=res_magx2 | 0x80;//se c<=limite
     }
     else {//quando b = 25,27,29,31 (high freq x2)
       res_magx2=res_magx2>>1;
       if(c<LIMITE_MIN_X2H)res_magx2=res_magx2 | 0x80;//se c<=limite
     }
   }

   res_magx1= res_magx1 & res_magx2;//faz um and lgico entre resultados x1 e x2

   /* res_magx1 contm o valor das 2 frequncias dtmf do sinal de entrada com as
   maiores magnitudes. A rotina abaixo pesquisa em uma tabela o valor da tecla
   apertada e aciona as respectivas sada q1 ... q4   */
   for(a=0;a<16;a++){//
     if(res_magx1 == tab_magnitudes[a]){//compara com tabela
       return (tab_teclas[a]);//retorna tecla apertada

     }
   }
   return 0;//tom no encontrado
}

//=============================================================================
//   INICIALIZAO DE PORTS E REGISTRADORES
//=============================================================================
void main (){
  //-----------------------------------------------------
  // PIC 16F676  definies de portas e registradores

  OPTION_REG=0B10000000; //timer 0 presc/2
  WPU=0;//sem pullups
  ANSEL = 0;   // todas os ports  digitais
  TRISA = 0b111010;  // port a direo dos pinos (in/out)
  TRISC = 0b000000;  // port c direo dos pinos (in/out)
  VRCON  = 0B10100000; //liga vref com 0 volts
  CMCON =  0B00000100;  // liga comparador interno - VREF=0 /NORMAL
  PORTC =   0b00000000;  // desliga sada
  ADCON0=0;//desliga conversor ADC
 // zera_array();//zera acumuladores de senos/cossenos
 // TMR0=0;
 // INTCON= 0B10100000; //liga interrupo do timer 0 /
  dispara_amostragens();//inicia a deteno de sinal dtmf no pino 12
//=============================================================================
//   LOOP PRINCIPAL DE REPETIO DO PROGRAMA
//=============================================================================
  for(;;){

      while(T0IE);//aguarda finalizao de amostragens
     tecla=decodifica_dtmf();//procura por tom dtmf


     //PARA USO COM TX/RX USE SOMENTE TECLAS QUE GEREM FREQUENCIAS BAIXAS

     if(tecla!=0){;//despreza recepo igual a zero

       if(!fvez){//le a primeira vez
        tecla_ant=tecla;
        fvez=1;
       }
       else{
        fvez=0;//le a segunda vez a mesma tecla em seguida
        if(tecla_ant==tecla){//confirma que recebeu a mesma tecla seguidamente

         switch (tecla){//agora vai escolher qual sada ser acionada e o modo

         case '1'://se tecla 1
         #ifdef tecla1_retencao //se foi optado pelo modo 'reteno'
         if(!fc1){
           if(can1)can1=0; else can1=1;//inverte a sada canal 1
           fc1=1;
         }
         #else//se foi optado por modo 'pulso'
         can1=1;//apenas liga a sada do canal 1
         #endif
         cnt1=0;//zera contador auxiliar de tempo
         break;


         case '2':
         #ifdef tecla2_retencao//se foi optado pelo modo 'reteno'
         if(!fc2){
           if(can2)can2=0; else can2=1;//apenas liga a sada do canal 2
           fc2=1;
         }
         #else//se foi optado por modo 'pulso'
         can2=1;//apenas liga a sada do canal 2
         #endif
         cnt2=0;//zera contador auxiliar de tempo
         break;



         case '3':
         #ifdef tecla3_retencao//se foi optado pelo modo 'reteno'
         if(!fc3){
           if(can3)can3=0; else can3=1;//apenas liga a sada do canal 3
           fc3=1;
         }
         #else//se foi optado por modo 'pulso'
         can3=1;//apenas liga a sada do canal 3
         #endif
         cnt3=0;//zera contador auxiliar de tempo
         break;


        case '4':
         #ifdef tecla4_retencao//se foi optado pelo modo 'reteno'
         if(!fc4){
           if(can4)can4=0; else can4=1;//apenas liga a sada do canal 4
           fc4=1;
         }
         #else//se foi optado por modo 'pulso'
         can4=1;//apenas liga a sada do canal 4
         #endif
         cnt4=0;//zera contador auxiliar de tempo
         break;

        case '5':
         #ifdef tecla5_retencao//se foi optado pelo modo 'reteno'
         if(!fc5){
           if(can5)can5=0; else can5=1;//apenas liga a sada do canal 5
           fc5=1;//apenas liga a sada do canal 5
         }
         #else
         can5=1;
         #endif//se foi optado por modo 'pulso'
         cnt5=0;//zera contador auxiliar de tempo
         break;

        case '6':
         #ifdef tecla6_retencao//se foi optado pelo modo 'reteno'
         if(!fc6){
           if(can6)can6=0; else can6=1;//apenas liga a sada do canal 6
           fc6=1;
         }
         #else
         can6=1;//apenas liga a sada do canal 6
         #endif//se foi optado por modo 'pulso'
         cnt6=0;//zera contador auxiliar de tempo
         break;

        case '7':
         #ifdef tecla7_retencao//se foi optado pelo modo 'reteno'
         if(!fc7){
           if(can7)can7=0; else can7=1;//apenas liga a sada do canal 7
           fc7=1;
         }
         #else//se foi optado por modo 'pulso'
         can7=1;//apenas liga a sada do canal *
         #endif
         cnt7=0;//zera contador auxiliar de tempo
         break;


        case '8':
         #ifdef tecla8_retencao//se foi optado pelo modo 'reteno'
         if(!fc8){
           if(can8)can8=0; else can8=1;//apenas liga a sada do canal 8
           fc8=1;
         }
         #else//se foi optado por modo 'pulso'
         can8=1;//apenas liga a sada do canal 0
         #endif
         cnt8=0;//zera contador auxiliar de tempo
         break;

         }//switch (tecla)
        }//if(tecla_ant==tecla)
      }//else
     }//if(tecla!=0)

     dispara_amostragens();//inicia processo de amostragens para capturar DTMF

     cnt1++;
     #ifndef tecla1_retencao //modo pulso
     if(cnt1>TEMPO_DESL_PULSO)can1=0;//desliga canal 1 depois de um tempo
     #else// modo reteno
     if(cnt1>TEMPO_DESL_RET)fc1=0;//apenas apaga o flag de controle do canal 1
     #endif


     cnt2++;
     #ifndef tecla2_retencao //modo pulso
     if(cnt2>TEMPO_DESL_PULSO)can2=0;//desliga canal 2 depois de um tempo
     #else // modo reteno
     if(cnt2>TEMPO_DESL_RET)fc2=0;//apenas apaga o flag de controle do canal 2
     #endif


     cnt3++;
     #ifndef tecla3_retencao //modo pulso
     if(cnt3>TEMPO_DESL_PULSO)can3=0;//desliga canal 3 depois de um tempo
     #else // modo reteno
     if(cnt3>TEMPO_DESL_RET)fc3=0;//apenas apaga o flag de controle do canal 3
     #endif

     cnt4++;
     #ifndef tecla4_retencao //modo pulso
     if(cnt4>TEMPO_DESL_PULSO)can4=0;//desliga canal 4 depois de um tempo
     #else // modo reteno
     if(cnt4>TEMPO_DESL_RET)fc4=0;//apenas apaga o flag de controle do canal 4
     #endif

     cnt5++;
     #ifndef tecla5_retencao //modo pulso
     if(cnt5>TEMPO_DESL_PULSO)can5=0;//desliga canal 5 depois de um tempo
     #else // modo reteno
     if(cnt5>TEMPO_DESL_RET)fc5=0;//apenas apaga o flag de controle do canal 5
     #endif

     cnt6++;
     #ifndef tecla6_retencao //modo pulso
     if(cnt6>TEMPO_DESL_PULSO)can6=0;//desliga canal 6 depois de um tempo
     #else // modo reteno
     if(cnt6>TEMPO_DESL_RET)fc6=0;//apenas apaga o flag de controle do canal 6
     #endif

     cnt7++;
     #ifndef tecla7_retencao //modo pulso
     if(cnt7>TEMPO_DESL_PULSO)can7=0;//desliga canal 7 depois de um tempo
     #else // modo reteno
     if(cnt7>TEMPO_DESL_RET)fc7=0;//apenas apaga o flag de controle do canal 7
     #endif

     cnt8++;
     #ifndef tecla8_retencao //modo pulso
     if(cnt8>TEMPO_DESL_PULSO)can8=0;//desliga canal 8 depois de um tempo
     #else // modo reteno
     if(cnt8>TEMPO_DESL_RET)fc8=0;//apenas apaga o flag de controle do canal 8
     #endif

  }//for(;;)
}//void main()
//-----------------------------------------------------------------------------


