/******************************************************************************
    TX CONTROLE REMOTO POR TONS DTMF

    C/ microcontrolador PIC16F676

    Transmissor para sinais DTMF com 8 canais acionados conforme a tecla DTMF.

    Direto, sem senhas - apertando uma tecla acionar a sada correspondente.
    
    Autor: Cludio Lrios

    Fontes de pesquisas:
    Baseado nos seguintes artigos publicados em Websites originalmente escritos
    em linguagem assembly  e sendo adaptado para linguagem C:

    Adaptaes de Tabelas e algoritimos Goertzel: Radu Constantinescu.
    Microchip Note Applications AN257 'DTMF Detection Using PIC18
    Microcontrollers' by Gaurang Kavaiya.
    DTMF Decoding Software with a PIC - by Wim de Vree PE1GRL - pe1grl@amsat.org
    Introduction: DTMF Touch Tone Decoder Using Microchip PIC Microprocessor by
    LargeMouthBass - https://www.instructables.com/member/LargeMouthBass/
    Black DTMF Decoding Algorithm - www.RomanBlack.com
    Agradecimentos a todos os autores destes artigos por compartilharem seus
    conhecimentos a todos que tem o desejo sincero de aprender.

    Compilador sugerido: CCS C V3.200


    PINAGEM DO PIC 16F676:

    PINO 12  =   SADA S2  (DTMF)
    PINO 10  =   boto tecla 1 (sada)
    PINO  9  =   boto tecla 2 (sada)
    PINO  8  =   boto tecla 4 (sada)
    PINO  7  =   boto tecla 5 (sada)
    PINO  6  =   boto tecla 7 (sada)
    PINO  5  =   boto tecla 8 (sada)
    PINO  13 =   boto tecla * (sada)
    PINO  11 =   SADA S1 (DTMF)
    PINO  3  =   boto tecla 0
    PINO  2  =   xtal 20MHZ
    PINO  3  =   xtal 20MHZ
    PINO  1  =   VCC
    PINO 14  =   GND

    Uso didtico

    Blog Picsource
******************************************************************************/

//==============================================================================
#include <16F676.h>
#use delay(clock=10000000)//frequencia do xtal
#fuses NOWDT,HS, NOCPD, NOPROTECT, NOMCLR, NOPUT, NOBROWNOUT

//bytes

#byte   FSR         = 0x04
#byte   PCLATH      = 0x0A
#byte   STATUS      = 0X03
#byte   TRISA       = 0x85
#byte   PORTA       = 0x05
#byte   TRISC       = 0x87
#byte   PORTC       = 0x07
#byte   TMR1H       = 0x0F
#byte   TMR1L       = 0x0E
#byte   TMR0        = 0X01
#byte   OPTION_REG  = 0x81
#byte   WPU         = 0x95
#byte   CMCON       = 0x19
#byte   ANSEL       = 0x91 //obs. diferente do pic12f675
#byte   ADRESH      = 0x1E
#byte   ADRESL      = 0x9E
#byte   T1CON       = 0X10
#byte   IOC         = 0x96
#byte   INTCON      = 0X0B
#byte   VRCON       = 0X99
#byte   ADCON0      = 0X1F
#byte   VRCON       = 0X99
#byte   FSR         = 0X04
#byte   INDF        = 0X00
#byte   PIR1        = 0X0C

//bit
#bit    GIE         = 0xb.7
#bit    PEIE        = 0xb.6
#bit    T0IE        = 0xb.5
#bit    TMR1IE      = 0x8C.0
#bit    TMR1IF      = 0xC.0
#bit    TMR1_ON     = 0x10.0
#bit    T0IF        = 0XB.2
#bit    TMR0_F0     = 0x01.0
#bit    C_OUT       = 0x19.6
#bit    GO_DONE     = 0X1F.1

//pinos de sada
#bit S1 = 0x5.2  //pino 11
#bit S2 = 0x5.1  //pino 12
//entradas
#bit bt1 = 0x7.0  //pino 10
#bit bt2 = 0x7.1  //pino  9
#bit bt3 = 0x7.2  //pino  8
#bit bt4 = 0x7.3  //pino  7
#bit bt5 = 0x7.4  //pino  6
#bit bt6 = 0x7.5  //pino  5
#bit bt7 = 0x5.0  //pino 13
#bit bt8 = 0x5.3  //pino 4

/*

        1        2        3        A   697 HZ
        4        5        6        B   770 HZ
        7        8        9        C   852 HZ
        *        0        #        D   941 HZ

       1209     1336     1477    1633


*/
/*TABELA H  - S1 - TIMER0  - PINO 11 (escala superior)
   1209 HZ= 128
   1336 HZ= 141
   1477 HZ= 151
   1633 HZ= 161
*/
const char tabela_h[]={//Valores de retorno para carga do timer0
// teclas 0   1   2   3   4   5   6   7   8   9   A   B   C   D   *   #
         141,128,141,151,128,141,151,128,141,151,161,161,161,161,128,151

};
/* TABELA L - S2 -TMR1 - PINO 12  (escala inferior)
    697 HZ= 33
    770 HZ= 54
    852 HZ= 74
    941 HZ= 92
*/
const char tabela_l[]={//Valores de retorno para carga do timer1
//teclas  0 1  2  3  4  5  6  7  8  9  A  B  C  D  *  #
         92,33,33,33,54,54,54,74,74,74,33,54,74,92,92,92

};


char FREQH,FREQL;


#int_global
void isr_timerX(){//interrupo do timer 0 e 1

char STATUS_TEMP,W_TEMP;
//char PCLATH_SAVE,FSR_SAVE;

   #ASM
   movwf	 W_TEMP
	swapf	 STATUS,W
	clrf	 STATUS		//ENSURE BANK0 SAVE
	movwf	 STATUS_TEMP
//	movf	 PCLATH,W	  //SAVE PCLATH
//	movwf  PCLATH_SAVE
//	movf	 FSR, W
//	movwf	 FSR_SAVE
  // #ENDASM
   BTFSC  PIR1,0 ; //TMR1IF
	GOTO PULA3
   BTFSS   INTCON,2
   GOTO EXIT
	BCF    INTCON,2 //FLAG TMR0 - FREQH - S1
	MOVF  FREQH,W
	MOVWF  TMR0
   BTFSS  S1
   GOTO P4
   BCF  S1
	GOTO EXIT
P4:
   BSF  S1
	GOTO EXIT

PULA3:
   BCF PIR1,0; //TMR1IF
	MOVF  FREQL,W
	MOVWF  TMR1L
	MOVLW  0XFF
	MOVWF  TMR1H
   BTFSS  S2
   GOTO P6
   BCF  S2
   GOTO EXIT
P6:
   BSF  S2
EXIT:

 // #asm //restabelece principais registradores com valores anteriores
//  	 movf	   FSR_SAVE,W
//	 movwf	FSR
//	 movf	   PCLATH_SAVE,W
//	 movwf	PCLATH		// RESTORE PCLATH
	 swapf	STATUS_TEMP,W
	 movwf	STATUS
	 swapf	W_TEMP,F
	 swapf	W_TEMP,W
  #endasm
}//fim da rotina de interrupo do timer 0

void enviar(char tecla){

  FREQH= tabela_h[tecla];
  FREQL= tabela_l[tecla];
  GIE=1;
}

//=============================================================================
//   INICIALIZAO DE PORTS E REGISTRADORES
//=============================================================================
void main (){
  //-----------------------------------------------------
  // PIC 16F676  definies de portas e registradores

  OPTION_REG = 0B10000010; //timer 0 presc/8
 // OPTION_REG = 0B10000011; //timer 0 presc/16
  WPU = 0;//sem pullups
  ANSEL = 0;   // todas os ports  digitais
  TRISA = 0b111001;  // port a direo dos pinos (in/out)
  TRISC = 0b111111;  // port c direo dos pinos (in/out)
  CMCON = 7;  // desliga comparador interno
  PORTA = 0b00000000;  // desliga sada
  ADCON0 = 0;//desliga conversor ADC
  T1CON=  0B00110000;
  T0IF = 0;//reseta flag de overflow do timer 0
  TMR1IF = 0;//reseta flag de overflow do timer 1
  TMR1IE = 1;//liga interrupo do timer 1
  PEIE=1;//liga interrupo de perifricos
  TMR1_ON = 1;//liga timer 1
  T0IE=1;                                                //DESCOMENTADO TESTEXXXXXXXXXXXXXX
//=============================================================================
//   LOOP PRINCIPAL DE REPETIO DO PROGRAMA
//=============================================================================
  for(;;){
  //testa se algum boto foi pressionado
     if(!bt1)enviar(1);
     else if(!bt2)enviar(2);
     else if(!bt3)enviar(3);
     else if(!bt4)enviar(4);
     else if(!bt5)enviar(5);
     else if(!bt6)enviar(6);
     else if(!bt7)enviar(7);
     else if(!bt8)enviar(8);
     else GIE=0;
  }//for(;;)
}//void main()
//-----------------------------------------------------------------------------


